@@ -3,7 +3,7 @@
  *
  * Copyright © 1995 Ian Jackson
  * Copyright © 2000, 2001 Wichert Akkerman
- * Copyright © 2006-2014 Guillem Jover <guillem@debian.org>
+ * Copyright © 2006-2015, 2017-2018 Guillem Jover <guillem@debian.org>
  * Copyright © 2011 Linaro Limited
  * Copyright © 2011 Raphaël Hertzog <hertzog@debian.org>
  *
@@ -45,14 +45,14 @@
 #include <dpkg/glob.h>
 #include <dpkg/buffer.h>
 #include <dpkg/options.h>
-
-#include "filesdb.h"
+#include <dpkg/db-fsys.h>
 
 
 static const char printforhelp[] = N_(
 "Use --help for help about diverting files.");
 
 static const char *admindir;
+const char *instdir;
 
 static bool opt_pkgname_match_any = true;
 static const char *opt_pkgname = NULL;
@@ -60,7 +60,7 @@ static const char *opt_divertto = NULL;
 
 static int opt_verbose = 1;
 static int opt_test = 0;
-static int opt_rename = 0;
+static int opt_rename = -1;
 
 
 static void
@@ -101,7 +101,10 @@ usage(const struct cmdinfo *cip, const char *value)
 "  --local                  all packages' versions are diverted.\n"
 "  --divert <divert-to>     the name used by other packages' versions.\n"
 "  --rename                 actually move the file aside (or back).\n"
+"  --no-rename              do not move the file aside (or back) (default).\n"
 "  --admindir <directory>   set the directory with the diversions file.\n"
+"  --instdir <directory>    set the root directory, but not the admin dir.\n"
+"  --root <directory>       set the directory of the root filesystem.\n"
 "  --test                   don't do anything, just demonstrate.\n"
 "  --quiet                  quiet operation, minimal output.\n"
 "  --help                   show this help message.\n"
@@ -118,6 +121,17 @@ usage(const struct cmdinfo *cip, const char *value)
 	exit(0);
 }
 
+static void
+opt_rename_setup(void)
+{
+	if (opt_rename >= 0)
+		return;
+
+	opt_rename = 0;
+	warning(_("please specify --no-rename explicitly, the default "
+	          "will change to --rename in 1.20.x"));
+}
+
 struct file {
 	const char *name;
 	enum {
@@ -131,7 +145,13 @@ struct file {
 static void
 file_init(struct file *f, const char *filename)
 {
-	f->name = filename;
+	struct varbuf usefilename = VARBUF_INIT;
+
+	varbuf_add_str(&usefilename, instdir);
+	varbuf_add_str(&usefilename, filename);
+	varbuf_end_str(&usefilename);
+
+	f->name = varbuf_detach(&usefilename);
 	f->stat_state = FILE_STAT_INVALID;
 }
 
@@ -159,7 +179,7 @@ check_writable_dir(struct file *f)
 	char *tmpname;
 	int tmpfd;
 
-	m_asprintf(&tmpname, "%s%s", f->name, ".dpkg-divert.tmp");
+	tmpname = str_fmt("%s%s", f->name, ".dpkg-divert.tmp");
 
 	tmpfd = creat(tmpname, 0600);
 	if (tmpfd < 0)
@@ -187,7 +207,7 @@ check_rename(struct file *src, struct file *dst)
 	 * just having +w is not enough, since people do mount things RO,
 	 * and we need to fail before we start mucking around with things.
 	 * So we open a file with the same name as the diversions but with
-	 * an extension that (hopefully) wont overwrite anything. If it
+	 * an extension that (hopefully) won't overwrite anything. If it
 	 * succeeds, we assume a writable filesystem.
 	 */
 
@@ -216,12 +236,12 @@ file_copy(const char *src, const char *dst)
 	if (srcfd < 0)
 		ohshite(_("unable to open file '%s'"), src);
 
-	m_asprintf(&tmp, "%s%s", dst, ".dpkg-divert.tmp");
+	tmp = str_fmt("%s%s", dst, ".dpkg-divert.tmp");
 	dstfd = creat(tmp, 0600);
 	if (dstfd < 0)
 		ohshite(_("unable to create file '%s'"), tmp);
 
-	push_cleanup(cu_filename, ~ehflag_normaltidy, NULL, 0, 1, tmp);
+	push_cleanup(cu_filename, ~ehflag_normaltidy, 1, tmp);
 
 	if (fd_fd_copy(srcfd, dstfd, -1, &err) < 0)
 		ohshit(_("cannot copy '%s' to '%s': %s"), src, tmp, err.str);
@@ -276,7 +296,7 @@ diversion_check_filename(const char *filename)
 }
 
 static const char *
-diversion_pkg_name(struct diversion *d)
+diversion_pkg_name(struct fsys_diversion *d)
 {
 	if (d->pkgset == NULL)
 		return ":";
@@ -329,7 +349,7 @@ diversion_current(const char *filename)
 }
 
 static const char *
-diversion_describe(struct diversion *d)
+diversion_describe(struct fsys_diversion *d)
 {
 	static struct varbuf str = VARBUF_INIT;
 	const char *pkgname;
@@ -356,17 +376,17 @@ divertdb_write(void)
 {
 	char *dbname;
 	struct atomic_file *file;
-	struct fileiterator *iter;
-	struct filenamenode *namenode;
+	struct fsys_hash_iter *iter;
+	struct fsys_namenode *namenode;
 
 	dbname = dpkg_db_get_path(DIVERSIONSFILE);
 
 	file = atomic_file_new(dbname, ATOMIC_FILE_BACKUP);
 	atomic_file_open(file);
 
-	iter = files_db_iter_new();
-	while ((namenode = files_db_iter_next(iter))) {
-		struct diversion *d = namenode->divert;
+	iter = fsys_hash_iter_new();
+	while ((namenode = fsys_hash_iter_next(iter))) {
+		struct fsys_diversion *d = namenode->divert;
 
 		if (d == NULL || d->useinstead == NULL)
 			continue;
@@ -376,7 +396,7 @@ divertdb_write(void)
 		        d->useinstead->name,
 		        diversion_pkg_name(d));
 	}
-	files_db_iter_free(iter);
+	fsys_hash_iter_free(iter);
 
 	atomic_file_sync(file);
 	atomic_file_close(file);
@@ -387,10 +407,37 @@ divertdb_write(void)
 }
 
 static bool
-diversion_is_owned_by_self(struct pkgset *set, struct filenamenode *namenode)
+diversion_is_essential(struct fsys_namenode *namenode)
 {
 	struct pkginfo *pkg;
-	struct filepackages_iterator *iter;
+	struct pkg_hash_iter *pkg_iter;
+	struct fsys_node_pkgs_iter *iter;
+	bool essential = false;
+
+	pkg_iter = pkg_hash_iter_new();
+	while ((pkg = pkg_hash_iter_next_pkg(pkg_iter))) {
+		if (pkg->installed.essential)
+			ensure_packagefiles_available(pkg);
+	}
+	pkg_hash_iter_free(pkg_iter);
+
+	iter = fsys_node_pkgs_iter_new(namenode);
+	while ((pkg = fsys_node_pkgs_iter_next(iter))) {
+		if (pkg->installed.essential) {
+			essential = true;
+			break;
+		}
+	}
+	fsys_node_pkgs_iter_free(iter);
+
+	return essential;
+}
+
+static bool
+diversion_is_owned_by_self(struct pkgset *set, struct fsys_namenode *namenode)
+{
+	struct pkginfo *pkg;
+	struct fsys_node_pkgs_iter *iter;
 	bool owned = false;
 
 	if (set == NULL)
@@ -399,14 +446,14 @@ diversion_is_owned_by_self(struct pkgset *set, struct filenamenode *namenode)
 	for (pkg = &set->pkg; pkg; pkg = pkg->arch_next)
 		ensure_packagefiles_available(pkg);
 
-	iter = filepackages_iter_new(namenode);
-	while ((pkg = filepackages_iter_next(iter))) {
+	iter = fsys_node_pkgs_iter_new(namenode);
+	while ((pkg = fsys_node_pkgs_iter_next(iter))) {
 		if (pkg->set == set) {
 			owned = true;
 			break;
 		}
 	}
-	filepackages_iter_free(iter);
+	fsys_node_pkgs_iter_free(iter);
 
 	return owned;
 }
@@ -416,11 +463,12 @@ diversion_add(const char *const *argv)
 {
 	const char *filename = argv[0];
 	struct file file_from, file_to;
-	struct diversion *contest, *altname;
-	struct filenamenode *fnn_from, *fnn_to;
+	struct fsys_diversion *contest, *altname;
+	struct fsys_namenode *fnn_from, *fnn_to;
 	struct pkgset *pkgset;
 
 	opt_pkgname_match_any = false;
+	opt_rename_setup();
 
 	/* Handle filename. */
 	if (!filename || argv[1])
@@ -435,28 +483,24 @@ diversion_add(const char *const *argv)
 	    S_ISDIR(file_from.stat.st_mode))
 		badusage(_("cannot divert directories"));
 
-	fnn_from = findnamenode(filename, 0);
+	fnn_from = fsys_hash_find_node(filename, 0);
 
 	/* Handle divertto. */
-	if (opt_divertto == NULL) {
-		char *str;
-
-		m_asprintf(&str, "%s.distrib", filename);
-		opt_divertto = str;
-	}
+	if (opt_divertto == NULL)
+		opt_divertto = str_fmt("%s.distrib", filename);
 
 	if (strcmp(filename, opt_divertto) == 0)
 		badusage(_("cannot divert file '%s' to itself"), filename);
 
 	file_init(&file_to, opt_divertto);
 
-	fnn_to = findnamenode(opt_divertto, 0);
+	fnn_to = fsys_hash_find_node(opt_divertto, 0);
 
 	/* Handle package name. */
 	if (opt_pkgname == NULL)
 		pkgset = NULL;
 	else
-		pkgset = pkg_db_find_set(opt_pkgname);
+		pkgset = pkg_hash_find_set(opt_pkgname);
 
 	/* Check we are not stomping over an existing diversion. */
 	if (fnn_from->divert || fnn_to->divert) {
@@ -468,7 +512,7 @@ diversion_add(const char *const *argv)
 			if (opt_verbose > 0)
 				printf(_("Leaving '%s'\n"),
 				       diversion_describe(fnn_from->divert));
-			exit(0);
+			return 0;
 		}
 
 		ohshit(_("'%s' clashes with '%s'"),
@@ -505,6 +549,9 @@ diversion_add(const char *const *argv)
 			       filename, pkgset->name);
 		opt_rename = false;
 	}
+	if (opt_rename && diversion_is_essential(fnn_from))
+		warning(_("diverting file '%s' from an Essential package with "
+		          "rename is dangerous, use --no-rename"), filename);
 	if (!opt_test) {
 		divertdb_write();
 		if (opt_rename)
@@ -515,12 +562,12 @@ diversion_add(const char *const *argv)
 }
 
 static bool
-diversion_is_shared(struct pkgset *set, struct filenamenode *namenode)
+diversion_is_shared(struct pkgset *set, struct fsys_namenode *namenode)
 {
 	const char *archname;
 	struct pkginfo *pkg;
 	struct dpkg_arch *arch;
-	struct filepackages_iterator *iter;
+	struct fsys_node_pkgs_iter *iter;
 	bool shared = false;
 
 	if (set == NULL)
@@ -534,14 +581,14 @@ diversion_is_shared(struct pkgset *set, struct filenamenode *namenode)
 	for (pkg = &set->pkg; pkg; pkg = pkg->arch_next)
 		ensure_packagefiles_available(pkg);
 
-	iter = filepackages_iter_new(namenode);
-	while ((pkg = filepackages_iter_next(iter))) {
+	iter = fsys_node_pkgs_iter_new(namenode);
+	while ((pkg = fsys_node_pkgs_iter_next(iter))) {
 		if (pkg->set == set && pkg->installed.arch != arch) {
 			shared = true;
 			break;
 		}
 	}
-	filepackages_iter_free(iter);
+	fsys_node_pkgs_iter_free(iter);
 
 	return shared;
 }
@@ -550,17 +597,19 @@ static int
 diversion_remove(const char *const *argv)
 {
 	const char *filename = argv[0];
-	struct filenamenode *namenode;
-	struct diversion *contest, *altname;
+	struct fsys_namenode *namenode;
+	struct fsys_diversion *contest, *altname;
 	struct file file_from, file_to;
 	struct pkgset *pkgset;
 
+	opt_rename_setup();
+
 	if (!filename || argv[1])
 		badusage(_("--%s needs a single argument"), cipaction->olong);
 
 	diversion_check_filename(filename);
 
-	namenode = findnamenode(filename, fnn_nonew);
+	namenode = fsys_hash_find_node(filename, FHFF_NONE);
 
 	if (namenode == NULL || namenode->divert == NULL ||
 	    namenode->divert->useinstead == NULL) {
@@ -573,7 +622,7 @@ diversion_remove(const char *const *argv)
 	if (opt_pkgname == NULL)
 		pkgset = NULL;
 	else
-		pkgset = pkg_db_find_set(opt_pkgname);
+		pkgset = pkg_hash_find_set(opt_pkgname);
 
 	contest = namenode->divert;
 	altname = contest->useinstead->divert;
@@ -599,7 +648,7 @@ diversion_remove(const char *const *argv)
 		if (opt_verbose > 0)
 			printf(_("Ignoring request to remove shared diversion '%s'.\n"),
 			       diversion_describe(contest));
-		exit(0);
+		return 0;
 	}
 
 	if (opt_verbose > 0)
@@ -626,8 +675,8 @@ diversion_remove(const char *const *argv)
 static int
 diversion_list(const char *const *argv)
 {
-	struct fileiterator *iter;
-	struct filenamenode *namenode;
+	struct fsys_hash_iter *iter;
+	struct fsys_namenode *namenode;
 	struct glob_node *glob_list = NULL;
 	const char *pattern;
 
@@ -637,14 +686,14 @@ diversion_list(const char *const *argv)
 	if (glob_list == NULL)
 		glob_list_prepend(&glob_list, m_strdup("*"));
 
-	iter = files_db_iter_new();
-	while ((namenode = files_db_iter_next(iter))) {
+	iter = fsys_hash_iter_new();
+	while ((namenode = fsys_hash_iter_next(iter))) {
 		struct glob_node *g;
-		struct diversion *contest = namenode->divert;
-		struct diversion *altname;
+		struct fsys_diversion *contest = namenode->divert;
+		struct fsys_diversion *altname;
 		const char *pkgname;
 
-		if (contest->useinstead == NULL)
+		if (contest == NULL || contest->useinstead == NULL)
 			continue;
 
 		altname = contest->useinstead->divert;
@@ -660,7 +709,7 @@ diversion_list(const char *const *argv)
 			}
 		}
 	}
-	files_db_iter_free(iter);
+	fsys_hash_iter_free(iter);
 
 	glob_list_free(glob_list);
 
@@ -671,17 +720,17 @@ static int
 diversion_truename(const char *const *argv)
 {
 	const char *filename = argv[0];
-	struct filenamenode *namenode;
+	struct fsys_namenode *namenode;
 
 	if (!filename || argv[1])
 		badusage(_("--%s needs a single argument"), cipaction->olong);
 
 	diversion_check_filename(filename);
 
-	namenode = findnamenode(filename, fnn_nonew);
+	namenode = fsys_hash_find_node(filename, FHFF_NONE);
 
 	/* Print the given name if file is not diverted. */
-	if (namenode && namenode->divert->useinstead)
+	if (namenode && namenode->divert && namenode->divert->useinstead)
 		printf("%s\n", namenode->divert->useinstead->name);
 	else
 		printf("%s\n", filename);
@@ -693,17 +742,17 @@ static int
 diversion_listpackage(const char *const *argv)
 {
 	const char *filename = argv[0];
-	struct filenamenode *namenode;
+	struct fsys_namenode *namenode;
 
 	if (!filename || argv[1])
 		badusage(_("--%s needs a single argument"), cipaction->olong);
 
 	diversion_check_filename(filename);
 
-	namenode = findnamenode(filename, fnn_nonew);
+	namenode = fsys_hash_find_node(filename, FHFF_NONE);
 
 	/* Print nothing if file is not diverted. */
-	if (namenode == NULL)
+	if (namenode == NULL || namenode->divert == NULL)
 		return 0;
 
 	if (namenode->divert->pkgset == NULL)
@@ -739,6 +788,19 @@ set_divertto(const struct cmdinfo *cip, const char *value)
 		badusage(_("divert-to may not contain newlines"));
 }
 
+static void
+set_instdir(const struct cmdinfo *cip, const char *value)
+{
+	instdir = dpkg_fsys_set_dir(value);
+}
+
+static void
+set_root(const struct cmdinfo *cip, const char *value)
+{
+	instdir = dpkg_fsys_set_dir(value);
+	admindir = dpkg_fsys_get_path(ADMINDIR);
+}
+
 static const struct cmdinfo cmdinfo_add =
 	ACTION("add",         0, 0, diversion_add);
 
@@ -750,11 +812,14 @@ static const struct cmdinfo cmdinfos[] = {
 	ACTION("truename",    0, 0, diversion_truename),
 
 	{ "admindir",   0,   1,  NULL,         &admindir, NULL          },
+	{ "instdir",    0,   1,  NULL,         NULL,      set_instdir,  0 },
+	{ "root",       0,   1,  NULL,         NULL,      set_root,     0 },
 	{ "divert",     0,   1,  NULL,         NULL,      set_divertto  },
 	{ "package",    0,   1,  NULL,         NULL,      set_package   },
 	{ "local",      0,   0,  NULL,         NULL,      set_package   },
 	{ "quiet",      0,   0,  &opt_verbose, NULL,      NULL, 0       },
 	{ "rename",     0,   0,  &opt_rename,  NULL,      NULL, 1       },
+	{ "no-rename",  0,   0,  &opt_rename,  NULL,      NULL, 0       },
 	{ "test",       0,   0,  &opt_test,    NULL,      NULL, 1       },
 	{ "help",      '?',  0,  NULL,         NULL,      usage         },
 	{ "version",    0,   0,  NULL,         NULL,      printversion  },
@@ -772,6 +837,7 @@ main(int argc, const char * const *argv)
 	dpkg_options_parse(&argv, cmdinfos, printforhelp);
 
 	admindir = dpkg_db_set_dir(admindir);
+	instdir = dpkg_fsys_set_dir(instdir);
 
 	env_pkgname = getenv("DPKG_MAINTSCRIPT_PACKAGE");
 	if (opt_pkgname_match_any && env_pkgname)
@@ -781,7 +847,7 @@ main(int argc, const char * const *argv)
 		setaction(&cmdinfo_add, NULL);
 
 	modstatdb_open(msdbrw_readonly);
-	filesdbinit();
+	fsys_hash_init();
 	ensure_diversions();
 
 	ret = cipaction->action(argv);
@@ -789,6 +855,7 @@ main(int argc, const char * const *argv)
 	modstatdb_shutdown();
 
 	dpkg_program_done();
+	dpkg_locales_done();
 
 	return ret;
 }
