@@ -2,7 +2,7 @@
  * dpkg - main program for package management
  * select.c - by-hand (rather than dselect-based) package selection
  *
- * Copyright © 1995,1996 Ian Jackson <ian@chiark.greenend.org.uk>
+ * Copyright © 1995,1996 Ian Jackson <ijackson@chiark.greenend.org.uk>
  * Copyright © 2006, 2008-2015 Guillem Jover <guillem@debian.org>
  * Copyright © 2011 Linaro Limited
  * Copyright © 2011 Raphaël Hertzog <hertzog@debian.org>
@@ -37,9 +37,9 @@
 #include <dpkg/pkg-show.h>
 #include <dpkg/pkg-spec.h>
 #include <dpkg/options.h>
+#include <dpkg/db-ctrl.h>
+#include <dpkg/db-fsys.h>
 
-#include "filesdb.h"
-#include "infodb.h"
 #include "main.h"
 
 static void getsel1package(struct pkginfo *pkg) {
@@ -67,7 +67,7 @@ getselections(const char *const *argv)
 
   modstatdb_open(msdbrw_readonly);
 
-  pkg_array_init_from_db(&array);
+  pkg_array_init_from_hash(&array);
   pkg_array_sort(&array, pkg_sorter_by_nonambig_name_arch);
 
   if (!*argv) {
@@ -103,6 +103,8 @@ getselections(const char *const *argv)
 
   pkg_array_destroy(&array);
 
+  modstatdb_shutdown();
+
   return 0;
 }
 
@@ -180,7 +182,8 @@ setselections(const char *const *argv)
     if (!pkg_is_informative(pkg, &pkg->installed) &&
         !pkg_is_informative(pkg, &pkg->available)) {
       db_possibly_outdated = true;
-      warning(_("package not in database at line %d: %.250s"), lno, namevb.buf);
+      warning(_("package not in status nor available database at line %d: %.250s"), lno, namevb.buf);
+      lno++;
       continue;
     }
 
@@ -199,7 +202,8 @@ setselections(const char *const *argv)
 
   if (db_possibly_outdated)
     warning(_("found unknown packages; this might mean the available database\n"
-              "is outdated, and needs to be updated through a frontend method"));
+              "is outdated, and needs to be updated through a frontend method;\n"
+              "please see the FAQ <https://wiki.debian.org/Teams/Dpkg/FAQ>"));
 
   return 0;
 }
@@ -208,7 +212,7 @@ int
 clearselections(const char *const *argv)
 {
   enum modstatdb_rw msdbflags;
-  struct pkgiterator *it;
+  struct pkg_hash_iter *iter;
   struct pkginfo *pkg;
 
   if (*argv)
@@ -222,12 +226,12 @@ clearselections(const char *const *argv)
   modstatdb_open(msdbflags);
   pkg_infodb_upgrade();
 
-  it = pkg_db_iter_new();
-  while ((pkg = pkg_db_iter_next_pkg(it))) {
-    if (!pkg->installed.essential)
+  iter = pkg_hash_iter_new();
+  while ((pkg = pkg_hash_iter_next_pkg(iter))) {
+    if (!pkg->installed.essential && pkg->want != PKG_WANT_UNKNOWN)
       pkg_set_want(pkg, PKG_WANT_DEINSTALL);
   }
-  pkg_db_iter_free(it);
+  pkg_hash_iter_free(iter);
 
   modstatdb_shutdown();
 
